if (!require("pacman")) install.packages("pacman")
pacman::p_load(tidyr, readr, dplyr, ggplot2, KernSmooth, shiny)
if(!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("cBioPortalData", force=TRUE)


library(tidyr)
library(readr)
library(ggplot2)
library(KernSmooth, include.only = 'bkde')
library(dplyr)

library(shiny)


#pull data from cbioportal--tmbsupp generated by the following code that is commented out to decrease memory usage in online version of app, where tmbsupp is read from a local file 

library(cBioPortalData)
tmbsupp<-clinicalData(cBioPortalData::cBioPortal(), "tmb_mskcc_2018")




tmbsupp$TMB_SCORE<-as.numeric(tmbsupp$TMB_NONSYNONYMOUS)
#round the TMB scores to nearest tenths
tmbsupp$TMB_SCORE <- round(tmbsupp$TMB_SCORE, digits = 1)

#set up grid of "true" mutation rates for the dbinom function
p_grid <- seq(from=0,to=0.0001999,0.0000001)
x_values<-seq(from=0, to = 199.9, 0.1)



# Define UI
ui <- fluidPage(
  titlePanel("Effect of Sequencing Assay Size on Uncertainty of TMB Calculation"),
sidebarLayout(
    sidebarPanel(  
      
      numericInput("num", 
                   h4("Choose TMB result"), 
                   value = 12, min = 0, max = 400),
    
      numericInput("assaysize", h4("Choose sequencing assay size (in Mb)"), 
                       value = 0.8, min = 0.2, max = 3, step = 0.1),
      selectInput("priorselect", h4("Choose TMB distribution to use for prior"), 
                  choices = list("No (flat) prior", "All tumors","Bladder Cancer", 
                                 "Breast Cancer", "Cancer of Unknown Primary", 
                                 "Colorectal Cancer", "Esophagogastric Cancer", 
                                 "Glioma", "Head and Neck Cancer", 
                                 "Melanoma", "Non-Small Cell Lung Cancer", 
                                 "Renal Cell Carcinoma" ), selected = "No (flat) prior"),

           
       radioButtons("percentile", h5("Display Histology-Specific TMB Percentiles"), choices = list("Do not display TMB percentiles"=1, "Display TMB percentiles"=2)),
      
       sliderInput("slider", h5("Adjust x axis maximum"),  min = 20, max = 200, value = 50)

),
    mainPanel(
      plotOutput("TMBplot", width = 900, height = 500),
      h5(textOutput("TMBtext")),
      h5(textOutput("quantiletext")),
      h5(textOutput("percentiletext"))
)
),
shiny::tags$div(p(""),
                  p("The accuracy of TMB calculation is dependent upon the number of megabases sequenced due to sampling error. 
                  This program calculates the middle 90% confidence interval for a TMB result based on a given assay size, and it 
                  calculates the percent chance that the true TMB result is greater than 10 mut/Mb. 
                  If relatively few megabases are sequenced, the true TMB value may be significantly affected by 
                  the ", em("a priori" ), "TMB distribution for the particular tumor type, and this effect can also be explored."),
                     p("This application only considers sampling error and does not consider additional inaccuracy due to other factors
                       (such as mutation call criteria, overrepresentation of hotspot mutations in small panels, etc.)"),
                     p("This application uses data from ", 
                       a(href="https://urldefense.com/v3/__https://doi.org/10.1038/s41588-018-0312-8__;!!NHLzug!dpOp7f890gLkNxGBoQ43lRm3oY2kRvO9V_0M0rCwIthkck0rWAYOqOF4KAYBcwYy$ ", "Samstein RM et al. Nat Genet. 2019 51:202-206,"), 
                       " publicly available through ", 
                       a(href="https://urldefense.com/v3/__https://www.cbioportal.org/study/summary?id=tmb_mskcc_2018*22,*22cBioPortal__;JSU!!NHLzug!dpOp7f890gLkNxGBoQ43lRm3oY2kRvO9V_0M0rCwIthkck0rWAYOqOF4KIwF4ohu$ ."),
                     ),
                ),
)



# Define server logic ----
server <- function(input, output) {

  
#get value for x axis maximum for plot
  xmax <- reactive({
    input$slider})
  
#get histology-specific prior TMB distributions, use bkde function to smooth  
  prior_reactive <- reactive(  {
  if (input$priorselect == "No (flat) prior")  { 
    rep(1,2000) 
    } else if (input$priorselect == "All tumors") {
    bkde(tmbsupp$TMB_SCORE, bandwidth=5, gridsize = 2000, range.x = c(0,200), truncate = FALSE)
      } else {
        tmb_reactive <- reactive(  {tmbsupp %>% filter(CANCER_TYPE == input$priorselect)})
        tmb <- tmb_reactive()
        bkde(tmb$TMB_SCORE, bandwidth=5, gridsize = 2000, range.x = c(0,200), truncate = FALSE)
      }
})
  
#normalize prior for plotting
  prior<-reactive(  {
    a<-prior_reactive()
    if (is.list(a)==TRUE) a<-abs(a$y) 
    a<-a /sum(a)
    a
  })
  
#get TMB deciles for plotting 
  TMBpercentile <- reactive(  {
    if (input$priorselect == "No (flat) prior" | input$priorselect == "All tumors")  { 
      quantile(tmbsupp$TMB_SCORE, probs = c(.1, .2, .3, .4, .5, .6, .7, .8, .9))
    } else {
      tmbpercentile_reactive <- reactive( {tmbsupp %>% filter(CANCER_TYPE == input$priorselect)})
      tmb2 <-tmbpercentile_reactive ()
      quantile(tmb2$TMB_SCORE, probs = c(.1, .2, .3, .4, .5, .6, .7, .8, .9))
    }
  })
  
#TMB decile dataframe generation
  tmbdf <- reactive({
    if (input$priorselect == "No (flat) prior" | input$priorselect == "All tumors")  { 
      q<-quantile(tmbsupp$TMB_SCORE, probs = c(.1, .2, .3, .4, .5, .6, .7, .8, .9))
      data.frame(xmin=c(0, q[1], q[2], q[3], q[4], q[5], q[6], q[7], q[8], q[9]),
                 xmax=c(ifelse(q[1]<xmax(), q[1], xmax()), ifelse(q[2]<xmax(), q[2], xmax()), 
                        ifelse(q[3]<xmax(), q[3], xmax()), ifelse(q[4]<xmax(), q[4], xmax()), 
                        ifelse(q[5]<xmax(), q[5], xmax()), ifelse(q[6]<xmax(), q[6], xmax()),
                        ifelse(q[7]<xmax(), q[7], xmax()), ifelse(q[8]<xmax(), q[8], xmax()),
                        ifelse(q[9]<xmax(), q[9], xmax()),  xmax()),
                        ymin = rep(0, 10), ymax=rep(Inf, 10), 
                        Deciles = c("0-10%", "10-20%", "20-30%", "30-40%", "40-50%", 
                                    "50-60%", "60-70%", "70-80%", "80-90%", "90-100%")
                 )
    } else {
      tmbpercentile_reactive <- reactive(  {tmbsupp %>% filter(CANCER_TYPE == input$priorselect)})
      tmb2 <-tmbpercentile_reactive ()
      q<-quantile(tmb2$TMB_SCORE, probs = c(.1, .2, .3, .4, .5, .6, .7, .8, .9))
      data.frame(xmin=c(0, q[1], q[2], q[3], q[4], q[5], q[6], q[7], q[8], q[9]),
                 xmax=c(ifelse(q[1]<xmax(), q[1], xmax()), ifelse(q[2]<xmax(), q[2], xmax()), 
                        ifelse(q[3]<xmax(), q[3], xmax()), ifelse(q[4]<xmax(), q[4], xmax()), 
                        ifelse(q[5]<xmax(), q[5], xmax()), ifelse(q[6]<xmax(), q[6], xmax()),
                        ifelse(q[7]<xmax(), q[7], xmax()), ifelse(q[8]<xmax(), q[8], xmax()),
                        ifelse(q[9]<xmax(), q[9], xmax()),  xmax()),
                 ymin = rep(0, 10), ymax=rep(Inf, 10), 
                 Deciles = c("0-10%", "10-20%", "20-30%", "30-40%", "40-50%", 
                             "50-60%", "60-70%", "70-80%", "80-90%", "90-100%")
      )
    }
  })



#get value for assay size
assaysize <- reactive(  {
  q <- input$assaysize * 1e6
  q

})
  
#get likely number of mutations detected for entered TMB value
mut_num <- reactive(  {
  q <- input$num * assaysize() / 1e6
  q <- round(q)
  q
})

#get likelihood
  likelihood <- reactive(  {dbinom(mut_num(), size = assaysize(), prob = p_grid )})
  norm_likelihood<- reactive(  {likelihood () / sum(likelihood())})
 
#get posterior, taking histology-specific prior distribution into account
  
  bayesprob <- reactive(  {likelihood() * prior()})
  posterior <- reactive(  {bayesprob() / sum(bayesprob())})

  
  #get data from posterior
  set.seed(100)
  
  samples <- reactive(  {sample(p_grid, prob = posterior(), size = 1e4, replace = TRUE)})
  
  probs <- reactive(  {quantile(samples(), probs = c(0.025, 0.05, 0.25, 0.5, 0.75, 0.9, 0.95, 0.975))})
  
#get 90% confidence interval for plotting
  q90 <- reactive(  {
    w <- quantile(samples(), probs = c(.05, 0.95))
    w <- w *1e6
    w
  })
#get likelihood TMB value is greater than 10 mut/Mb for text
  g10 <-reactive ({
    g <- sum(samples()>0.000010)
    g <- round((g/1e2), 0)
    g
  })

#get percentiles corresponding to 5% and 95% for text  
  tmb90 <- reactive({
    if (input$priorselect == "No (flat) prior" | input$priorselect == "All tumors")  { 
      q<-round(ecdf(tmbsupp$TMB_SCORE)(q90()[1])*100, 0)
      r<-round(ecdf(tmbsupp$TMB_SCORE)(q90()[2])*100, 0)
      c(q,r)
    } else {
      tmbpercentile_reactive <- reactive(  {tmbsupp %>% filter(CANCER_TYPE == input$priorselect)})
      tmb2 <-tmbpercentile_reactive ()
      q<-round(ecdf(tmb2$TMB_SCORE)(q90()[1])*100, 0)
      r<-round(ecdf(tmb2$TMB_SCORE)(q90()[2])*100, 0)
      c(q,r)
    }
  })
  
  
#combine data together into dataframe for plotting
  x<-reactive(  {cbind(prior(), norm_likelihood(), posterior(), x_values)})
  df <- reactive(  {
    b<-x()
    b<-as.data.frame(b)
    names(b)<-c("Prior", "Likelihood", "Posterior", "TMB")
    b<-pivot_longer(b, c("Prior", "Likelihood", "Posterior"), names_to="Distribution", values_to="Density")
    b
    })
  
 
  

# make the plot
  line_types <- c("Posterior"=1,"Likelihood"=2, "Prior"=3)
  
  output$TMBplot <- renderPlot({if(input$percentile==2){
    ggplot()+
      geom_rect(data = tmbdf(), aes(xmin=xmin, ymin=ymin, xmax=xmax, ymax=ymax, fill = Deciles), alpha=0.5)+
      scale_fill_viridis_d()+
      geom_line(data = df(), mapping = aes(TMB, Density, linetype=Distribution), color = "black")+
      geom_ribbon(data = df() %>% filter(TMB > q90()[1] & TMB < q90()[2] & Distribution=="Posterior"),
                  aes(x=TMB, ymin = 0, ymax = Density),alpha =0.5) +
      xlim(0,xmax())+theme_minimal(base_size = 20)+scale_y_continuous(labels = NULL)+

      scale_linetype_manual(name = "Distribution", values=line_types, labels=c("Posterior (takes prior\ndistribution into account)","Likelihood\n(raw test result)", "Prior"))+
      theme(legend.position = "right")+labs(fill="Histology-specific\npercentiles", legend.text = element_text(size=15))+labs(x="TMB (mut/Mb)")
    
  }else{
    ggplot()+
      geom_line(data = df(), mapping = aes(TMB, Density, linetype=Distribution), color = "black")+
      geom_ribbon(data = df() %>% filter(TMB > q90()[1] & TMB < q90()[2] & Distribution=="Posterior"),
                  aes(x=TMB, ymin = 0, ymax = Density),alpha =0.5) +
      xlim(0,xmax())+theme_minimal(base_size = 20)+scale_y_continuous(labels = NULL)+
      scale_linetype_manual(name = "Distribution", values=line_types, 
                            labels=c("Posterior (takes prior\ndistribution into account)", "Likelihood\n(raw test result)", "Prior"))+
      theme(legend.position = "right", legend.text = element_text(size=15)) +labs(x="TMB (mut/Mb)")
    
      }
    })
static_num<-reactive( {input$num})
static_assay<-reactive({input$assaysize})
static_prior <- reactive( {
  p<-input$priorselect
  p<-tolower(p)
  p
})
  
# get reported mutation number for text below the plot
  rptnum <- reactive({round(mut_num()/assaysize()*1e6, digits=1)})

    output$TMBtext  <- renderText({
    paste("If sequencing ", (static_assay())," Mb, ", (mut_num()), " detected mutations would yield a reported TMB value closest to ", (static_num())," mut/Mb (", rptnum()," mut/Mb).", sep="")
  })
    output$quantiletext  <- renderText({if (static_prior()=="no (flat) prior"){
      
      paste("Without considering prior information about the distribution of TMB, the middle 90% confidence interval for this result ranges from ", (q90()[1])," to ",(q90()[2])," mut/Mb (shaded region in plot). The chance that the true TMB is greater than 10 mut/Mb is ",(g10()),"%.", sep="")
    }
     else
      {
      paste("Taking into account the distribution of TMB for ", (static_prior()),", the middle 90% confidence interval for this result ranges from ", (q90()[1])," to ",(q90()[2])," mut/Mb (shaded region in plot). The chance that the true TMB is greater than 10 mut/Mb is ",(g10()),"%.", sep="")
        
      }
   })
    
    output$percentiletext  <- renderText({if(input$percentile==2){
      
      if (static_prior()=="no (flat) prior"){
      
      paste("For all tumors, this corresponds to a percentile range of ", (tmb90()[1]), "% to ", (tmb90()[2]), "%.", sep="")
      } else
      {
        paste("For ", (static_prior()),", this corresponds to a percentile range of ", (tmb90()[1]), "% to ", (tmb90()[2]), "%.", sep="") 
      }
    } else NULL
    })
}
# Run the app
shinyApp(ui = ui, server = server)
  
  